library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
library work;
use work.all;

ENTITY quad_top IS
generic (WIDTH   : integer := 16);
    PORT (
        CHA  : IN std_logic;
        CHB  : IN std_logic;
        CLK  : IN std_logic;
        RST  : IN std_logic;
        OE   : IN std_logic;
		SEL  : IN std_logic;
		inh : in std_logic;
 		CNT  :OUT std_logic;
		UP_Down :OUT std_logic;
        D    : OUT std_logic_vector (7 downto 0)
    );
END quad_top;

ARCHITECTURE behavior OF quad_top IS

signal filt_cha,filt_chb : std_logic;

signal t_count : std_logic_vector(15 downto 0);
signal data_out : std_logic_vector(7 downto 0);
signal cnt_en : std_logic;
signal up_ndown : std_logic;


COMPONENT digital_filter
    PORT (
      CHA: IN std_logic;
	  CHB:  IN std_logic;
      CLK:    IN std_logic;
      Rst:   IN std_logic;
	  CHA_filt : OUT std_logic;
      CHB_filt:    OUT std_logic
           );
END COMPONENT;

COMPONENT Quad_decoder 
    PORT (
      filt_A : IN std_logic;
	  filt_B:      IN std_logic;
      Clk:    IN std_logic;
      rst:   IN std_logic;
      Count:    OUT std_logic;
      Updown: OUT std_logic
    );
END COMPONENT;

COMPONENT Position_counter 
    PORT(	
        rst       : IN std_logic;
        CLK      : IN std_logic;
        ENABLE   : IN std_logic;
        UP_DOWN  : IN std_logic;
        Q_count  : OUT std_logic_vector (15 downto 0)
       
    );
END COMPONENT ;

COMPONENT position_latch 
    PORT (
        DATA   : IN std_logic_vector (15 downto 0);
        RST  : IN std_logic; 
        CLK    : IN std_logic;
        OE     : IN std_logic;
	SEL    : IN std_logic;
	INH    : IN std_logic;
        Q : OUT std_logic_vector(7 downto 0)
    );
END COMPONENT ;

BEGIN

U1: Digital_Filter

PORT MAP(
  CHA	  => CHA,
  CHB	  => CHB,
  CLK   => CLK,
  RST   => RST,
  CHA_filt =>filt_cha,
  CHB_filt => filt_chb
  );

U2: Quad_decoder 
    PORT MAP (
      filt_A =>filt_cha,
      filt_B  =>filt_chb,
      Clk => clk,
      rst=> rst,
	  count =>cnt_en,
	  updown =>up_ndown
      );
      
U3: Position_counter 
    PORT MAP(	
        rst   =>rst,
        CLK     =>clk,
        ENABLE   => cnt_en,
        UP_DOWN  =>up_ndown,
		Q_count  =>t_count
        );
        
U4: Position_latch 
    PORT MAP (
	 DATA   => t_count,
	 RST  => rst,
	 CLK    => clk,
	 OE     => oe,
	 SEL    => sel,
	 INH    => inh,
	 Q => Data_out
	   );

  CNT <= cnt_en;
  up_down <= up_ndown;
  D <= data_out;
 end behavior;
